/* quaternion/test.c
 * 
 * Copyright (C) 2006 Frdric-Emmanuel PICCA
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

#include <config.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <gsl/gsl_ieee_utils.h>
#include <gsl/gsl_test.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_quaternion.h>
#include <gsl/gsl_quaternion_math.h>

int
test_quaternion_constructor1(void)
{
  int stat = 0;
  gsl_quaternion q;
  
  /* Check values. */
  q = GSL_QUATERNION_ZERO;
  if (GSL_QUATERNION_REAL(q) != 0) stat++;
  if (GSL_QUATERNION_IMAG(q) != 0) stat++;
  if (GSL_QUATERNION_JMAG(q) != 0) stat++;
  if (GSL_QUATERNION_KMAG(q) != 0) stat++;
  
  return stat;
}
  
int
test_quaternion_constructor2(void)
{
  int stat = 0;
  gsl_quaternion q;
  
  /* Check values. */
  q = GSL_QUATERNION_ONE;
  if (GSL_QUATERNION_REAL(q) != 1) stat++;
  if (GSL_QUATERNION_IMAG(q) != 0) stat++;
  if (GSL_QUATERNION_JMAG(q) != 0) stat++;
  if (GSL_QUATERNION_KMAG(q) != 0) stat++;
  
  return stat;
}
  
int
test_quaternion_constructor3(void)
{
  int stat = 0;
  gsl_quaternion q;
  
  /* Check values. */
  q = GSL_QUATERNION_NEGONE;
  if (GSL_QUATERNION_REAL(q) != -1) stat++;
  if (GSL_QUATERNION_IMAG(q) != 0) stat++;
  if (GSL_QUATERNION_JMAG(q) != 0) stat++;
  if (GSL_QUATERNION_KMAG(q) != 0) stat++;
  
  return stat;
}
 
int
test_quaternion_constructor4(void)
{
  int stat = 0;
  gsl_quaternion q;
  
  /* Check values. */
  q = gsl_quaternion_rect(1, 2, 3, 4);
  if (GSL_QUATERNION_REAL(q) != 1) stat++;
  if (GSL_QUATERNION_IMAG(q) != 2) stat++;
  if (GSL_QUATERNION_JMAG(q) != 3) stat++;
  if (GSL_QUATERNION_KMAG(q) != 4) stat++;
  
  return stat;
}
  
int
test_quaternion_constructor5(void)
{
  int stat = 0;
  
  /* Check values. */
  gsl_vector * v = gsl_vector_alloc (3);
  gsl_vector_set (v, 0, 1);
  gsl_vector_set (v, 1, -1);
  gsl_vector_set (v, 2, .5);
  
  gsl_quaternion q = gsl_quaternion_polar (M_PI_2, v);
  //printf("%f %f %f %f\n", GSL_QUATERNION_REAL (q), GSL_QUATERNION_IMAG (q), GSL_QUATERNION_JMAG (q), GSL_QUATERNION_KMAG (q));
  if (fabs (GSL_QUATERNION_REAL (q) - M_SQRT1_2) > 1e-14) stat++;
  if (fabs (GSL_QUATERNION_IMAG (q) - 0.471404520791032) > 1e-14) stat++;
  if (fabs (GSL_QUATERNION_JMAG (q) + 0.471404520791032) > 1e-14) stat++;
  if (fabs (GSL_QUATERNION_KMAG (q) - 0.235702260395515) > 1e-14) stat++;
  gsl_vector_free (v);
 
  return stat;
}

int
test_quaternion_abs(void)
{
  int stat = 0;
  gsl_quaternion q = gsl_quaternion_rect(1., 2., 3., 4.);
  
  /* Check values. */
  if (fabs(gsl_quaternion_abs(q) - 5.4772255750516609) > 1e-14) stat++;
  
  return stat;
}

int
test_quaternion_arg(void)
{
  int stat = 0;
  gsl_quaternion q = gsl_quaternion_rect(1., 2., 3., 4.);
  
  /* Check values. */
  if (fabs(gsl_quaternion_arg(q) - 1.3871923165159781) > 1e-14) stat++;
  //printf("%f/n", gsl_quaternion_arg(q) * 10000000000);
  
  return stat;
}

int
test_quaternion_add(void)
{
    int stat = 0;
    gsl_quaternion qref = gsl_quaternion_rect(2., 3., 4., 5.);
    gsl_quaternion q1 = gsl_quaternion_rect(1., 2., 3., 4.);
    gsl_quaternion q2 = gsl_quaternion_rect(1., 1., 1., 1.);
    gsl_quaternion q = gsl_quaternion_add(q1, q2);

    if (!GSL_QUATERNION_EQ(qref, q)) stat++;

    return stat;
}

int
test_quaternion_add_real(void)
{
    int stat = 0;
    gsl_quaternion qref = gsl_quaternion_rect(2., 2., 3., 4.);
    gsl_quaternion q1 = gsl_quaternion_rect(1., 2., 3., 4.);
    gsl_quaternion q = gsl_quaternion_add_real(q1, 1.);

    if (!GSL_QUATERNION_EQ(qref, q)) stat++;

    return stat;
}

int
test_quaternion_add_imag(void)
{
    int stat = 0;
    gsl_quaternion qref = gsl_quaternion_rect(1., 3., 3., 4.);
    gsl_quaternion q1 = gsl_quaternion_rect(1., 2., 3., 4.);
    gsl_quaternion q = gsl_quaternion_add_imag(q1, 1.);

    if (!GSL_QUATERNION_EQ(qref, q)) stat++;

    return stat;
}

int
test_quaternion_add_jmag(void)
{
    int stat = 0;
    gsl_quaternion qref = gsl_quaternion_rect(1., 2., 4., 4.);
    gsl_quaternion q1 = gsl_quaternion_rect(1., 2., 3., 4.);
    gsl_quaternion q = gsl_quaternion_add_jmag(q1, 1.);

    if (!GSL_QUATERNION_EQ(qref, q)) stat++;

    return stat;
}

int
test_quaternion_add_kmag(void)
{
    int stat = 0;
    gsl_quaternion qref = gsl_quaternion_rect(1., 2., 3., 5.);
    gsl_quaternion q1 = gsl_quaternion_rect(1., 2., 3., 4.);
    gsl_quaternion q = gsl_quaternion_add_kmag(q1, 1.);

    if (!GSL_QUATERNION_EQ(qref, q)) stat++;

    return stat;
}

int
test_quaternion_sub(void)
{
    int stat = 0;
    gsl_quaternion qref = gsl_quaternion_rect(0., 1., 2., 3.);
    gsl_quaternion q1 = gsl_quaternion_rect(1., 2., 3., 4.);
    gsl_quaternion q2 = gsl_quaternion_rect(1., 1., 1., 1.);
    gsl_quaternion q = gsl_quaternion_sub(q1, q2);

    if (!GSL_QUATERNION_EQ(qref, q)) stat++;

    return stat;
}

int
test_quaternion_sub_real(void)
{
    int stat = 0;
    gsl_quaternion qref = gsl_quaternion_rect(0., 2., 3., 4.);
    gsl_quaternion q1 = gsl_quaternion_rect(1., 2., 3., 4.);
    gsl_quaternion q = gsl_quaternion_sub_real(q1, 1.);

    if (!GSL_QUATERNION_EQ(qref, q)) stat++;

    return stat;
}

int
test_quaternion_sub_imag(void)
{
    int stat = 0;
    gsl_quaternion qref = gsl_quaternion_rect(1., 1., 3., 4.);
    gsl_quaternion q1 = gsl_quaternion_rect(1., 2., 3., 4.);
    gsl_quaternion q = gsl_quaternion_sub_imag(q1, 1.);

    if (!GSL_QUATERNION_EQ(qref, q)) stat++;

    return stat;
}

int
test_quaternion_sub_jmag(void)
{
    int stat = 0;
    gsl_quaternion qref = gsl_quaternion_rect(1., 2., 2., 4.);
    gsl_quaternion q1 = gsl_quaternion_rect(1., 2., 3., 4.);
    gsl_quaternion q = gsl_quaternion_sub_jmag(q1, 1.);

    if (!GSL_QUATERNION_EQ(qref, q)) stat++;

    return stat;
}

int
test_quaternion_sub_kmag(void)
{
    int stat = 0;
    gsl_quaternion qref = gsl_quaternion_rect(1., 2., 3., 3.);
    gsl_quaternion q1 = gsl_quaternion_rect(1., 2., 3., 4.);
    gsl_quaternion q = gsl_quaternion_sub_kmag(q1, 1.);

    if (!GSL_QUATERNION_EQ(qref, q)) stat++;

    return stat;
}

int
test_quaternion_mul(void)
{
    int stat = 0;
    gsl_quaternion qref = gsl_quaternion_rect(-28., 4., 6., 8.);
    gsl_quaternion q1 = gsl_quaternion_rect(1., 2., 3., 4.);
    gsl_quaternion q = gsl_quaternion_mul(q1, q1);

    
    if (!GSL_QUATERNION_EQ(qref, q)) stat++;

    return stat;
}

int
test_quaternion_mul_real(void)
{
    int stat = 0;
    gsl_quaternion qref = gsl_quaternion_rect(2., 4., 6., 8.);
    gsl_quaternion q1 = gsl_quaternion_rect(1., 2., 3., 4.);
    gsl_quaternion q = gsl_quaternion_mul_real(q1, 2.);

    if (!GSL_QUATERNION_EQ(qref, q)) stat++;

    return stat;
}

int
test_quaternion_mul_imag(void)
{
    int stat = 0;
    gsl_quaternion qref = gsl_quaternion_rect(-4., 2., 8., -6.);
    gsl_quaternion q1 = gsl_quaternion_rect(1., 2., 3., 4.);
    gsl_quaternion q = gsl_quaternion_mul_imag(q1, 2.);

    if (!GSL_QUATERNION_EQ(qref, q)) stat++;

    return stat;
}

int
test_quaternion_mul_jmag(void)
{
    int stat = 0;
    gsl_quaternion qref = gsl_quaternion_rect(-6., -8., 2., 4.);
    gsl_quaternion q1 = gsl_quaternion_rect(1., 2., 3., 4.);
    gsl_quaternion q = gsl_quaternion_mul_jmag(q1, 2.);

    if (!GSL_QUATERNION_EQ(qref, q)) stat++;

    return stat;
}

int
test_quaternion_mul_kmag(void)
{
    int stat = 0;
    gsl_quaternion qref = gsl_quaternion_rect(-8., 6., -4., 2.);
    gsl_quaternion q1 = gsl_quaternion_rect(1., 2., 3., 4.);
    gsl_quaternion q = gsl_quaternion_mul_kmag(q1, 2.);

    if (!GSL_QUATERNION_EQ(qref, q)) stat++;

    return stat;
}

int
test_quaternion_div_real(void)
{
    int stat = 0;
    gsl_quaternion qref = gsl_quaternion_rect(.5, 1., 1.5, 2.);
    gsl_quaternion q1 = gsl_quaternion_rect(1., 2., 3., 4.);
    gsl_quaternion q = gsl_quaternion_div_real(q1, 2.);

    if (!GSL_QUATERNION_EQ(qref, q)) stat++;

    return stat;
}

int
test_quaternion_conjugate(void)
{
    int stat = 0;
    gsl_quaternion qref = gsl_quaternion_rect(1., -2., -3., -4);
    gsl_quaternion q1 = gsl_quaternion_rect(1., 2., 3., 4.);
    gsl_quaternion q = gsl_quaternion_conjugate(q1);

    if (!GSL_QUATERNION_EQ(qref, q)) stat++;

    return stat;
}

int
test_quaternion_neg(void)
{
    int stat = 0;
    gsl_quaternion qref = gsl_quaternion_rect(-1., -2., -3., -4.);
    gsl_quaternion q1 = gsl_quaternion_rect(1., 2., 3., 4.);
    gsl_quaternion q = gsl_quaternion_neg(q1);

    if (!GSL_QUATERNION_EQ(qref, q)) stat++;

    return stat;
}

int
test_quaternion_invert(void)
{
    int stat = 0;
    gsl_quaternion qref = gsl_quaternion_rect(1./30., -1./15., -1./10., -2./15.);
    gsl_quaternion q1 = gsl_quaternion_rect(1., 2., 3., 4.);
    
    gsl_quaternion q = gsl_quaternion_invert(q1);

    if (!GSL_QUATERNION_EQ(qref, q)) stat++;

    return stat;
}

int main()
{
  gsl_ieee_env_setup ();

  gsl_test( test_quaternion_constructor1(), "constructor1 QUATERNION");
  gsl_test( test_quaternion_constructor2(), "constructor2 QUATERNION");
  gsl_test( test_quaternion_constructor3(), "constructor3 QUATERNION");
  gsl_test( test_quaternion_constructor4(), "constructor4 QUATERNION");
  gsl_test( test_quaternion_constructor5(), "constructor5 QUATERNION");
  gsl_test( test_quaternion_abs(),          "Abs QUATERNION");
  gsl_test( test_quaternion_arg(),          "Arg QUATERNION");
  gsl_test( test_quaternion_add(),          "Add QUATERNION");
  gsl_test( test_quaternion_add_real(),     "Add_real QUATERNION");
  gsl_test( test_quaternion_add_imag(),     "Add_imag QUATERNION");
  gsl_test( test_quaternion_add_jmag(),     "Add_jmag QUATERNION");
  gsl_test( test_quaternion_add_kmag(),     "Add_kmag QUATERNION");
  gsl_test( test_quaternion_sub(),          "Sub QUATERNION");
  gsl_test( test_quaternion_sub_real(),     "Sub_real QUATERNION");
  gsl_test( test_quaternion_sub_imag(),     "Sub_imag QUATERNION");
  gsl_test( test_quaternion_sub_jmag(),     "Sub_jmag QUATERNION");
  gsl_test( test_quaternion_sub_kmag(),     "Sub_kmag QUATERNION");
  gsl_test( test_quaternion_mul(),          "Mul QUATERNION");
  gsl_test( test_quaternion_mul_real(),     "Mul_real QUATERNION");
  gsl_test( test_quaternion_mul_imag(),     "Mul_imag QUATERNION");
  gsl_test( test_quaternion_mul_jmag(),     "Mul_jmag QUATERNION");
  gsl_test( test_quaternion_mul_kmag(),     "Mul_kmag QUATERNION");
  gsl_test( test_quaternion_div_real(),     "Div_real QUATERNION");
  gsl_test( test_quaternion_conjugate(),    "Conjugate QUATERNION");
  gsl_test( test_quaternion_neg(),          "neg QUATERNION");
  gsl_test( test_quaternion_invert(),       "Invert QUATERNION");
  exit (gsl_test_summary());
}
